@@ -5,7 +5,6 @@ use std::collections::{HashMap, HashSet};
 use rustc_hash::FxHashSet;
 
 use crate::{
-    hunks::{compact_gaps, ensure_contiguous},
     lines::LineNumber,
     syntax::{zip_repeat_shorter, MatchKind, MatchedPos},
 };
@@ -20,10 +19,16 @@ pub const MAX_PADDING: usize = 3;
 pub fn all_matched_lines_filled(
     lhs_mps: &[MatchedPos],
     rhs_mps: &[MatchedPos],
+    lhs_lines: &[&str],
+    rhs_lines: &[&str],
 ) -> Vec<(Option<LineNumber>, Option<LineNumber>)> {
     let matched_lines = all_matched_lines(lhs_mps, rhs_mps);
 
-    compact_gaps(&ensure_contiguous(&matched_lines))
+    compact_gaps(&ensure_contiguous(&match_preceding_blanks(
+        &matched_lines,
+        lhs_lines,
+        rhs_lines,
+    )))
 }
 
 fn all_matched_lines(
@@ -121,6 +126,126 @@ fn merge_in_opposite_lines(
     res
 }
 
+/// If the lines immediately before `current` are blank on both sides,
+/// return the line numbers of those blank lines.
+fn match_blanks_between(
+    current: (LineNumber, LineNumber),
+    prev: (LineNumber, LineNumber),
+    lhs_lines: &[&str],
+    rhs_lines: &[&str],
+) -> Vec<(Option<LineNumber>, Option<LineNumber>)> {
+    let (mut current_lhs, mut current_rhs) = current;
+    if current_lhs.0 == 0 || current_rhs.0 == 0 {
+        return vec![];
+    }
+
+    current_lhs = (current_lhs.0 - 1).into();
+    current_rhs = (current_rhs.0 - 1).into();
+
+    let mut res = vec![];
+    while current_lhs > prev.0 && current_rhs > prev.1 {
+        if lhs_lines[current_lhs.0] == "" && rhs_lines[current_rhs.0] == "" {
+            res.push((Some(current_lhs), Some(current_rhs)));
+
+            current_lhs = (current_lhs.0 - 1).into();
+            current_rhs = (current_rhs.0 - 1).into();
+        } else {
+            break;
+        }
+    }
+
+    res.reverse();
+    res
+}
+
+fn match_blanks_before(
+    current: (LineNumber, LineNumber),
+    lhs_lines: &[&str],
+    rhs_lines: &[&str],
+) -> Vec<(Option<LineNumber>, Option<LineNumber>)> {
+    let (mut current_lhs, mut current_rhs) = current;
+    if current_lhs.0 == 0 || current_rhs.0 == 0 {
+        return vec![];
+    }
+
+    current_lhs = (current_lhs.0 - 1).into();
+    current_rhs = (current_rhs.0 - 1).into();
+
+    let mut res = vec![];
+    loop {
+        if lhs_lines[current_lhs.0] == "" && rhs_lines[current_rhs.0] == "" {
+            res.push((Some(current_lhs), Some(current_rhs)));
+
+            if current_lhs.0 == 0 || current_rhs.0 == 0 {
+                break;
+            }
+
+            current_lhs = (current_lhs.0 - 1).into();
+            current_rhs = (current_rhs.0 - 1).into();
+        } else {
+            break;
+        }
+    }
+
+    res.reverse();
+    res
+}
+
+/// For every line number pair, if there are blank lines preceding the
+/// pair, add those blank lines to the vec.
+///
+/// This substantially improves alignment, leading to more readable
+/// diffs.
+///
+/// We don't need to match blank lines following each pair. After
+/// matching up all the matching lines, we just display the remaining
+/// lines side-by-side regardless of content (see
+/// `ensure_contiguous`). If there are blank lines immediately
+/// following the pair, they will get aligned by this.
+fn match_preceding_blanks(
+    line_nums: &[(Option<LineNumber>, Option<LineNumber>)],
+    lhs_lines: &[&str],
+    rhs_lines: &[&str],
+) -> Vec<(Option<LineNumber>, Option<LineNumber>)> {
+    let mut prev_lhs = None;
+    let mut prev_rhs = None;
+
+    let mut res: Vec<(Option<LineNumber>, Option<LineNumber>)> = vec![];
+    for (lhs_line_num, rhs_line_num) in line_nums {
+        match (lhs_line_num, rhs_line_num, prev_lhs, prev_rhs) {
+            (Some(lhs_line_num), Some(rhs_line_num), None, None) => {
+                // The first pair.
+                res.append(&mut match_blanks_before(
+                    (*lhs_line_num, *rhs_line_num),
+                    lhs_lines,
+                    rhs_lines,
+                ));
+            }
+            (Some(lhs_line_num), Some(rhs_line_num), Some(prev_lhs), Some(prev_rhs)) => {
+                // Later pairs.
+                res.append(&mut match_blanks_between(
+                    (*lhs_line_num, *rhs_line_num),
+                    (prev_lhs, prev_rhs),
+                    lhs_lines,
+                    rhs_lines,
+                ));
+            }
+            _ => {}
+        }
+
+        res.push((*lhs_line_num, *rhs_line_num));
+
+        if lhs_line_num.is_some() {
+            prev_lhs = *lhs_line_num;
+        }
+        if rhs_line_num.is_some() {
+            prev_rhs = *rhs_line_num;
+        }
+    }
+
+    res
+}
+
 // TODO: use FxHashMap here.
 pub fn opposite_positions(mps: &[MatchedPos]) -> HashMap<LineNumber, HashSet<LineNumber>> {
     let mut res: HashMap<LineNumber, HashSet<LineNumber>> = HashMap::new();
@@ -404,6 +529,120 @@ pub fn add_context(
         .collect()
 }
 
+/// Ensure that we don't miss any intermediate values.
+///
+/// Before:
+///
+/// ```text
+/// 1 11
+/// 3 14
+/// 4 --
+/// ```
+///
+/// After:
+///
+/// ```text
+/// 1 10
+/// 2 12 (choosing to align even though content doesn't match)
+/// - 13 (fix uneven gap)
+/// 3 14
+/// 4 -- (preserve outer gaps)
+/// ```
+fn ensure_contiguous(
+    lines: &[(Option<LineNumber>, Option<LineNumber>)],
+) -> Vec<(Option<LineNumber>, Option<LineNumber>)> {
+    let mut res: Vec<(Option<LineNumber>, Option<LineNumber>)> = vec![];
+
+    let mut lhs_min: Option<LineNumber> = None;
+    let mut rhs_min: Option<LineNumber> = None;
+
+    for (lhs_line, rhs_line) in lines {
+        if let Some(lhs_line) = lhs_line {
+            if let Some(lhs_prev_min) = lhs_min {
+                if lhs_prev_min.0 + 1 != lhs_line.0 {
+                    res.extend((lhs_prev_min.0 + 1..lhs_line.0).map(|l| (Some(l.into()), None)));
+                }
+            }
+            lhs_min = Some(*lhs_line);
+        }
+        if let Some(rhs_line) = rhs_line {
+            if let Some(rhs_prev_min) = rhs_min {
+                if rhs_prev_min.0 + 1 != rhs_line.0 {
+                    res.extend((rhs_prev_min.0 + 1..rhs_line.0).map(|r| (None, Some(r.into()))));
+                }
+            }
+            rhs_min = Some(*rhs_line);
+        }
+        res.push((*lhs_line, *rhs_line));
+    }
+
+    res
+}
+
+/// Before:
+///
+/// 10 --
+/// 11 --
+/// -- 20
+/// 12 21
+///
+/// After:
+///
+/// 10 20
+/// 11 --
+/// 12 21
+///
+/// The returned vec will contain no (None, None) pairs.
+fn compact_gaps(
+    items: &[(Option<LineNumber>, Option<LineNumber>)],
+) -> Vec<(Option<LineNumber>, Option<LineNumber>)> {
+    let mut res: Vec<(Option<LineNumber>, Option<LineNumber>)> = vec![];
+    // A vec of the most recent single-sided lines, e.g.
+    //
+    // 10 --
+    // 11 --
+    //
+    // All items must be Some on the same side.
+    let mut unpaired_lines: Vec<(Option<LineNumber>, Option<LineNumber>)> = vec![];
+
+    for (lhs_line, rhs_line) in items {
+        match (lhs_line, rhs_line) {
+            (Some(lhs_line), None) => {
+                match unpaired_lines.first() {
+                    Some((None, Some(rhs_line))) => {
+                        // We've found a line that can be compacted.
+                        res.push((Some(*lhs_line), Some(*rhs_line)));
+                        unpaired_lines.remove(0);
+                    }
+                    _ => {
+                        unpaired_lines.push((Some(*lhs_line), None));
+                    }
+                }
+            }
+            (None, Some(rhs_line)) => {
+                match unpaired_lines.first() {
+                    Some((Some(lhs_line), None)) => {
+                        // We've found a line that can be compacted.
+                        res.push((Some(*lhs_line), Some(*rhs_line)));
+                        unpaired_lines.remove(0);
+                    }
+                    _ => {
+                        unpaired_lines.push((None, Some(*rhs_line)));
+                    }
+                }
+            }
+            _ => {
+                res.extend(unpaired_lines);
+                unpaired_lines = vec![];
+                res.push((*lhs_line, *rhs_line));
+            }
+        }
+    }
+
+    res.extend(unpaired_lines);
+    res
+}
+
 #[cfg(test)]
 mod tests {
     use std::iter::FromIterator;
@@ -575,4 +814,98 @@ mod tests {
             ]
         );
     }
+
+    #[test]
+    fn test_compact_gaps() {
+        let res = compact_gaps(&[
+            (Some(0.into()), None),
+            (None, Some(0.into())),
+            (Some(1.into()), Some(1.into())),
+        ]);
+        assert_eq!(
+            res,
+            vec![
+                (Some(0.into()), Some(0.into())),
+                (Some(1.into()), Some(1.into())),
+            ]
+        )
+    }
+
+    #[test]
+    fn test_compact_gaps_with_larger_gap() {
+        let res = compact_gaps(&[
+            (Some(0.into()), None),
+            (Some(1.into()), None),
+            (None, Some(0.into())),
+            (None, Some(1.into())),
+        ]);
+        assert_eq!(
+            res,
+            vec![
+                (Some(0.into()), Some(0.into())),
+                (Some(1.into()), Some(1.into())),
+            ]
+        )
+    }
+
+    #[test]
+    fn test_match_preceding_blanks() {
+        let lhs_lines = vec!["x", "", "", "y"];
+        let rhs_lines = vec!["x", "extra", "", "", "y"];
+
+        let res = match_preceding_blanks(
+            &[
+                (Some(0.into()), Some(0.into())),
+                (Some(3.into()), Some(4.into())),
+            ],
+            &lhs_lines,
+            &rhs_lines,
+        );
+        assert_eq!(
+            res,
+            vec![
+                (Some(0.into()), Some(0.into())),
+                (Some(1.into()), Some(2.into())),
+                (Some(2.into()), Some(3.into())),
+                (Some(3.into()), Some(4.into())),
+            ]
+        );
+    }
+
+    #[test]
+    fn test_match_preceding_blanks_first_pair() {
+        let lhs_lines = vec!["", "", "y"];
+        let rhs_lines = vec!["extra", "", "", "y"];
+
+        let res =
+            match_preceding_blanks(&[(Some(2.into()), Some(3.into()))], &lhs_lines, &rhs_lines);
+        assert_eq!(
+            res,
+            vec![
+                (Some(0.into()), Some(1.into())),
+                (Some(1.into()), Some(2.into())),
+                (Some(2.into()), Some(3.into())),
+            ]
+        );
+    }
+
+    #[test]
+    fn test_match_blanks_between() {
+        let lhs_lines = vec!["x", "", "", "y"];
+        let rhs_lines = vec!["x", "extra", "", "", "y"];
+
+        let res = match_blanks_between(
+            (3.into(), 4.into()),
+            (0.into(), 0.into()),
+            &lhs_lines,
+            &rhs_lines,
+        );
+        assert_eq!(
+            res,
+            vec![
+                (Some(1.into()), Some(2.into())),
+                (Some(2.into()), Some(3.into()))
+            ]
+        );
+    }
 }
