@@ -428,91 +428,6 @@ mod tests {
         );
     }
 
-    #[test]
-    fn test_novel_tree() {
-        let arena = Arena::new();
-
-        let lhs = vec![Syntax::new_list(
-            &arena,
-            "[",
-            pos_helper(0),
-            vec![
-                Syntax::new_atom(&arena, pos_helper(1), "1", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(2), "2", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(3), "3", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(4), "4", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(5), "5", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(6), "6", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(7), "7", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(8), "8", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(9), "9", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(10), "10", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(11), "11", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(12), "12", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(13), "13", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(14), "14", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(15), "15", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(16), "16", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(17), "17", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(18), "18", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(19), "19", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(20), "20", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(21), "21", AtomKind::Normal),
-            ],
-            "]",
-            pos_helper(100),
-        )];
-
-        let rhs = vec![Syntax::new_list(
-            &arena,
-            "[",
-            pos_helper(0),
-            vec![
-                Syntax::new_atom(&arena, pos_helper(1), "d1", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(2), "d2", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(3), "d3", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(4), "d4", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(5), "d5", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(6), "d6", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(7), "d7", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(8), "d8", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(9), "d9", AtomKind::Normal),
-                // This is the only common atom:
-                Syntax::new_atom(&arena, pos_helper(10), "10", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(11), "d11", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(12), "d12", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(13), "d13", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(14), "d14", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(15), "d15", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(16), "d16", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(17), "d17", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(18), "d18", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(19), "d19", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(20), "d20", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(21), "d21", AtomKind::Normal),
-            ],
-            "]",
-            pos_helper(100),
-        )];
-        init_all_info(&lhs, &rhs);
-
-        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
-        let route = shortest_path(start);
-
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
-        assert_eq!(
-            actions,
-            vec![
-                NovelTreeRHS {
-                    num_descendants: 21
-                },
-                NovelTreeLHS {
-                    num_descendants: 21
-                },
-            ]
-        );
-    }
-
     #[test]
     fn replace_similar_comment() {
         let arena = Arena::new();
