@@ -7,6 +7,11 @@
 // This tends to trigger on larger tuples of simple types, and naming
 // them would probably be worse for readability.
 #![allow(clippy::type_complexity)]
+// == "" is often clearer when dealing with strings.
+#![allow(clippy::comparison_to_empty)]
+// It's common to have pairs foo_lhs and foo_rhs, leading to double
+// the number of arguments and triggering this lint.
+#![allow(clippy::too_many_arguments)]
 
 mod context;
 mod dijkstra;
@@ -17,6 +22,7 @@ mod hunks;
 mod inline;
 mod line_parser;
 mod lines;
+mod myers_diff;
 mod options;
 mod positions;
 mod side_by_side;
@@ -39,7 +45,7 @@ use mimalloc::MiMalloc;
 
 /// The global allocator used by difftastic.
 ///
-/// Diffing allocates a large amount of memory, and MiMalloc performs
+/// Diffing allocates a large amount of memory, and `MiMalloc` performs
 /// better.
 #[global_allocator]
 static GLOBAL: MiMalloc = MiMalloc;
@@ -89,7 +95,7 @@ fn main() {
             match guess(path, &src) {
                 Some(lang) => {
                     let ts_lang = tsp::from_language(lang);
-                    let (tree, _) = tsp::parse_to_tree(&src, &ts_lang);
+                    let tree = tsp::parse_to_tree(&src, &ts_lang);
                     tsp::print_tree(&src, &tree);
                 }
                 None => {
@@ -415,9 +421,9 @@ fn print_diff_result(
                     style::header(&summary.path, 1, 1, "binary", use_color, background)
                 );
                 if changed {
-                    println!("Binary contents changed.")
+                    println!("Binary contents changed.");
                 } else {
-                    println!("No changes.")
+                    println!("No changes.");
                 }
             }
         }
@@ -427,7 +433,7 @@ fn print_diff_result(
                 "{}",
                 style::header(&summary.path, 1, 1, "binary", use_color, background)
             );
-            println!("Binary contents changed.")
+            println!("Binary contents changed.");
         }
     }
 }
