@@ -13,8 +13,10 @@ pub fn mark_unchanged<'a>(
     split_unchanged(&lhs_nodes, &rhs_nodes)
 }
 
+#[derive(Debug)]
 enum ChangeState {
-    Unchanged,
+    UnchangedDelimiter,
+    UnchangedNode,
     PossiblyChanged,
 }
 
@@ -36,7 +38,16 @@ fn split_unchanged<'a>(
         split_unchanged_toplevel(lhs_nodes, rhs_nodes, size_threshold)
     {
         match cs {
-            ChangeState::Unchanged => {
+            ChangeState::UnchangedDelimiter => {
+                assert_eq!(lhs_section_nodes.len(), rhs_section_nodes.len());
+                for (lhs_section_node, rhs_section_node) in
+                    lhs_section_nodes.iter().zip(rhs_section_nodes.iter())
+                {
+                    lhs_section_node.set_change(ChangeKind::Unchanged(rhs_section_node));
+                    rhs_section_node.set_change(ChangeKind::Unchanged(lhs_section_node));
+                }
+            }
+            ChangeState::UnchangedNode => {
                 assert_eq!(lhs_section_nodes.len(), rhs_section_nodes.len());
                 for (lhs_section_node, rhs_section_node) in
                     lhs_section_nodes.iter().zip(rhs_section_nodes.iter())
@@ -108,6 +119,11 @@ fn split_unchanged_toplevel<'a>(
                                     size_threshold,
                                 );
                                 if split_children.len() > 1 {
+                                    res.push((
+                                        ChangeState::UnchangedDelimiter,
+                                        section_lhs_nodes,
+                                        section_rhs_nodes,
+                                    ));
                                     // Managed to further split.
                                     res.append(&mut split_children);
                                 } else {
@@ -133,7 +149,7 @@ fn split_unchanged_toplevel<'a>(
                         section_rhs_nodes = vec![];
                     }
 
-                    res.push((ChangeState::Unchanged, vec![lhs_node], vec![rhs_node]));
+                    res.push((ChangeState::UnchangedNode, vec![lhs_node], vec![rhs_node]));
                 }
             }
             myers_diff::DiffResult::Left(lhs) => {
@@ -146,11 +162,37 @@ fn split_unchanged_toplevel<'a>(
     }
 
     if !section_lhs_nodes.is_empty() || !section_rhs_nodes.is_empty() {
-        res.push((
-            ChangeState::PossiblyChanged,
-            section_lhs_nodes,
-            section_rhs_nodes,
-        ));
+        // TODO: there's a lot of duplication with the loop above.
+        match as_singleton_list_children(&section_lhs_nodes, &section_rhs_nodes) {
+            Some((lhs_children, rhs_children)) => {
+                let mut split_children =
+                    split_unchanged_toplevel(&lhs_children, &rhs_children, size_threshold);
+                if split_children.len() > 1 {
+                    res.push((
+                        ChangeState::UnchangedDelimiter,
+                        section_lhs_nodes,
+                        section_rhs_nodes,
+                    ));
+                    // Managed to further split.
+                    res.append(&mut split_children);
+                } else {
+                    // Did not split further. Keep the outer list, so we can use
+                    // its delimiter when doing the tree diff.
+                    res.push((
+                        ChangeState::PossiblyChanged,
+                        section_lhs_nodes,
+                        section_rhs_nodes,
+                    ));
+                }
+            }
+            None => {
+                res.push((
+                    ChangeState::PossiblyChanged,
+                    section_lhs_nodes,
+                    section_rhs_nodes,
+                ));
+            }
+        }
     }
 
     res
@@ -362,7 +404,7 @@ mod tests {
     }
 
     #[test]
-    fn test_mark_unchanged_toplevel_at_start() {
+    fn test_split_unchanged_toplevel_at_start() {
         let arena = Arena::new();
         let config = from_language(guess_language::Language::EmacsLisp);
 
@@ -390,7 +432,7 @@ mod tests {
     }
 
     #[test]
-    fn test_mark_unchanged_toplevel_at_end() {
+    fn test_split_unchanged_toplevel_at_end() {
         let arena = Arena::new();
         let config = from_language(guess_language::Language::EmacsLisp);
 
@@ -417,7 +459,7 @@ mod tests {
     }
 
     #[test]
-    fn test_mark_preserves_outer_delimiters() {
+    fn test_split_preserves_outer_delimiters() {
         let arena = Arena::new();
         let config = from_language(guess_language::Language::EmacsLisp);
 
@@ -443,7 +485,7 @@ mod tests {
     }
 
     #[test]
-    fn test_mark_unchanged_middle() {
+    fn test_split_unchanged_middle() {
         let arena = Arena::new();
         let config = from_language(guess_language::Language::EmacsLisp);
 
@@ -479,7 +521,7 @@ mod tests {
     }
 
     #[test]
-    fn test_mark_unchanged_multiple() {
+    fn test_split_unchanged_multiple() {
         let arena = Arena::new();
         let config = from_language(guess_language::Language::EmacsLisp);
 
@@ -504,4 +546,31 @@ mod tests {
             ]
         );
     }
+
+    #[test]
+    fn test_split_unchanged_outer_delimiter() {
+        let arena = Arena::new();
+        let config = from_language(guess_language::Language::EmacsLisp);
+
+        let lhs_nodes = parse(
+            &arena,
+            "(novel-lhs-before (1 2 3 4 5 6 7 8 9 10) novel-lhs-after)",
+            &config,
+        );
+        let rhs_nodes = parse(
+            &arena,
+            "(novel-rhs-before (1 2 3 4 5 6 7 8 9 10) novel-rhs-after)",
+            &config,
+        );
+        init_all_info(&lhs_nodes, &rhs_nodes);
+
+        let res = split_unchanged(&lhs_nodes, &rhs_nodes);
+        dbg!(&res);
+        assert_eq!(res.len(), 2);
+
+        assert_eq!(
+            lhs_nodes[0].change(),
+            Some(ChangeKind::Unchanged(rhs_nodes[0]))
+        );
+    }
 }
